# -*- coding:utf-8 -*-
"""
zopePublisherHotfix 2008-06-09 0.1

Modify getPreferredCharsets method in class HTTPCharsets of 
zope/publisher/http.py  

This fixes http://dev.plone.org/plone/ticket/8185
Problem Description:
Setting a plone site with MS IE 6/7 or Safari 3, Japanese 
site name and description doesn't display correctly, turn 
into garbage characters as screen image as following link. 
I guess this problem depend on formlib.


Author: Shimizukawa(http://www.freia.jp/taka) 
        Terada(http://www.cmscom.jp/)
"""

from zope.publisher.http import sort_charsets
from zope.i18n.interfaces import IUserPreferredCharsets
from ZPublisher import Converters

from zope.publisher.http import HTTPCharsets

def getPreferredCharsets(self):
    '''See interface IUserPreferredCharsets'''
    charsets = []
    sawstar = sawiso88591 = 0
    header_present = 'HTTP_ACCEPT_CHARSET' in self.request
    for charset in self.request.get('HTTP_ACCEPT_CHARSET', '').split(','):
        charset = charset.strip().lower()
        if charset:
            if ';' in charset:
                charset, quality = charset.split(';')
                if not quality.startswith('q='):
                    # not a quality parameter
                    quality = 1.0
                else:
                    try:
                        quality = float(quality[2:])
                    except ValueError:
                        continue
            else:
                quality = 1.0
            if quality == 0.0:
                continue
            if charset == '*':
                sawstar = 1
            if charset == 'iso-8859-1':
                sawiso88591 = 1
            charsets.append((quality, charset))
    # Quoting RFC 2616, $14.2: If no "*" is present in an Accept-Charset
    # field, then all character sets not explicitly mentioned get a
    # quality value of 0, except for ISO-8859-1, which gets a quality
    # value of 1 if not explicitly mentioned.
    # And quoting RFC 2616, $14.2: "If no Accept-Charset header is
    # present, the default is that any character set is acceptable."
    if not sawstar and not sawiso88591 and header_present:
        ## Modifid by Shimizukawa
        charsets.append((1.0, Converters.default_encoding))
        ## end
    # UTF-8 is **always** preferred over anything else.
    # Reason: UTF-8 is not specific and can encode the entire unicode
    # range , unlike many other encodings. Since Zope can easily use very
    # different ranges, like providing a French-Chinese dictionary, it is
    # always good to use UTF-8.
    charsets.sort(sort_charsets)
    charsets = [charset for quality, charset in charsets]
    if sawstar and 'utf-8' not in charsets:
        charsets.insert(0, 'utf-8')
    return charsets


HTTPCharsets.getPreferredCharsets = getPreferredCharsets
